<?php

declare(strict_types=1);

namespace App\Components\Admin\Service\Transformer;

use App\Entity\Language;
use App\Entity\Point;
use App\Entity\PointLangData;
use Doctrine\ORM\EntityManagerInterface;
use App\Entity\Interfaces\PointDataFieldsInterface as Fields;
use App\Entity\Interfaces\PointLangDataFieldsInterface as LangFields;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

class PointFormDataTransformer implements TransformerInterface
{
    private EntityManagerInterface $entityManager;

    /**
     * @param EntityManagerInterface $entityManager
     */
    public  function __construct(EntityManagerInterface $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    /**
     * @param Point $point
     *
     * @return array
     *
     * @throws BadRequestHttpException
     */
    public function transformFromEntity($point): array
    {
        if (!($point instanceof Point)) {
            throw new BadRequestHttpException('First argument must be Point entity');
        }

        $pointExtData = $point->getData();

        $data = [
            'city' => $point->getCity(),
            'type' => $point->getType(),
            'is_active' => $point->getIsActive(),
            'lat' => $point->getLat(),
            'lon' => $point->getLon(),
            'logo' => $point->getLogo(),
            'url' => $point->getUrl(),
            'email' => $pointExtData[Fields::FIELD_EMAIl] ?? '',
            'website' => $pointExtData[Fields::FIELD_WEBSITE] ?? '',
            'phones' => $pointExtData[Fields::FIELD_PHONES] ?? '',
            'comment' => $pointExtData[Fields::FIELD_COMMENT] ?? '',
            'services' => $pointExtData[Fields::FIELD_SERVICES] ?? [],
            'detailed_type' => $pointExtData[Fields::FIELD_DETAILED_TYPE] ?? '',
        ];

        foreach ($point->getPointLangData() as $pointLangData) {
            $langId = $pointLangData->getLanguage()->getId();
            $pointLangExtData = $pointLangData->getData();

            $data['lang_' . $langId . '_title'] = $pointLangData->getTitle();
            $data['lang_' . $langId . '_desc'] = $pointLangExtData[LangFields::FIELD_DESC ?? ''];
            $data['lang_' . $langId . '_short_desc'] = $pointLangExtData[LangFields::FIELD_SHORT_DESC ?? ''];
            $data['lang_' . $langId . '_address'] = $pointLangExtData[LangFields::FIELD_ADDRESS ?? ''];
        }

        return $data;
    }

    /**
     * @param array $form
     * @param Point $point
     *
     * @throws BadRequestHttpException
     */
    public function transformToEntity(array $form, $point): void
    {
        if (!($point instanceof Point)) {
            throw new BadRequestHttpException('Second argument must be Point entity');
        }

        $pointExtData = [
            Fields::FIELD_EMAIl => (string) ($form['email'] ?? ''),
            Fields::FIELD_WEBSITE => (string) ($form['website'] ?? ''),
            Fields::FIELD_PHONES => (string) ($form['phones'] ?? ''),
            Fields::FIELD_COMMENT => (string) ($form['comment'] ?? ''),
            Fields::FIELD_SERVICES => (array) ($form['services'] ?? []),
            Fields::FIELD_DETAILED_TYPE => (string) ($form['detailed_type'] ?? ''),
        ];

        $point->setLat((string) ($form['lat'] ?? ''))
            ->setLon((string) ($form['lon'] ?? ''))
            ->setLogo((string) ($form['logo'] ?? ''))
            ->setUrl($form['url'] ?? null)
            ->setIsActive((bool) ($form['is_active'] ?? true))
            ->setCity($form['city'] ?? null)
            ->setType($form['type'] ?? null)
            ->setData($pointExtData);

        $languages = $this->entityManager->getRepository(Language::class)->findBy(['is_active' => true]);

        $pointLangDataRepository = $this->entityManager->getRepository(PointLangData::class);

        foreach ($languages as $language) {
            $pointLangData = $pointLangDataRepository->findOneBy([
                'language' => $language,
                'point' => $point,
            ]);

            if ($pointLangData === null) {
                $pointLangData = (new PointLangData())
                    ->setPoint($point)
                    ->setLanguage($language);

                $point->addPointLangData($pointLangData);
            }

            $pointLangData->setTitle((string) ($form['lang_' . $language->getId() . '_title'] ?? ''));

            $pointLangExtData = [
                LangFields::FIELD_DESC => (string) ($form['lang_' . $language->getId() . '_desc'] ?? ''),
                LangFields::FIELD_SHORT_DESC => (string) ($form['lang_' . $language->getId() . '_short_desc'] ?? ''),
                LangFields::FIELD_ADDRESS => (string) ($form['lang_' . $language->getId() . '_address'] ?? ''),
            ];

            $pointLangData->setData($pointLangExtData);
        }
    }
}